(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3);
    global.stackedgroupedbar = mod.exports;
  }
})(this, function (exports, _react, _d) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var margin = {
    top: 40,
    right: 10,
    bottom: 20,
    left: 10
  };

  var StackedGroupedBar = function (_React$Component) {
    _inherits(StackedGroupedBar, _React$Component);

    function StackedGroupedBar(props) {
      _classCallCheck(this, StackedGroupedBar);

      var _this = _possibleConstructorReturn(this, (StackedGroupedBar.__proto__ || Object.getPrototypeOf(StackedGroupedBar)).call(this, props));

      _this.state = {
        innerWidth: _this.props.width - margin.left - margin.right,
        innerHeight: _this.props.height - margin.top - margin.bottom,
        dataSet: null

      };

      return _this;
    }

    _createClass(StackedGroupedBar, [{
      key: 'componentWillMount',
      value: function componentWillMount() {
        this.setUpData();
      }
    }, {
      key: 'setUpData',
      value: function setUpData() {
        var m,
            n,
            xz,
            yz,
            y01z,
            yMax,
            y1Max,
            dataSet,
            transposedData = [],
            data = this.props.data;
        m = data['x-ticks'] || 25;
        xz = data['x-axis'] || d3.range(m);
        yz = data.data;
        n = Object.keys(yz).length;

        for (var i = 0; i < m; i++) {
          var entry = {};
          for (var key in yz) {
            entry[key] = yz[key][i];
          }
          transposedData.push(entry);
        }

        y01z = d3.stack().keys(Object.keys(yz)).order(d3.stackOrderNone).offset(d3.stackOffsetNone)(transposedData);
        yMax = d3.max(Object.keys(yz), function (y) {
          return d3.max(yz[y]);
        });
        y1Max = d3.max(y01z, function (y) {
          return d3.max(y, function (d) {
            return d[1];
          });
        });
        dataSet = {
          m: m,
          n: n,
          xz: xz,
          yz: yz,
          y01z: y01z,
          yMax: yMax,
          y1Max: y1Max
        };

        this.setState({ dataSet: dataSet });
      }
    }, {
      key: 'componentDidMount',
      value: function componentDidMount() {
        var svg,
            g,
            x,
            y,
            color,
            series,
            rect,
            timeout,
            data = this.state.dataSet;

        svg = d3.select('.stackedgrouped');
        g = svg.append('g').attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');
        x = d3.scaleBand().domain(d3.range(data.m)).rangeRound([0, this.state.innerWidth]).padding(0.08);
        y = d3.scaleLinear().domain([0, data.y1Max]).range([this.state.innerHeight, 0]);
        color = d3.scaleOrdinal().domain(d3.range(data.n)).range(d3.schemeCategory20c);
        series = g.selectAll('.series').data(data.y01z).enter().append('g').attr('fill', function (d, i) {
          return color(i);
        }).attr('id', function (d) {
          return d.index;
        });
        rect = series.selectAll('rect').data(function (d) {
          return d;
        }).enter().append('rect').attr('x', function (d, i) {
          return x(i);
        }).attr('y', this.state.innerHeight).attr('width', x.bandwidth()).attr('height', 0);

        rect.transition().delay(function (d, i) {
          return i * 10;
        }).attr('y', function (d) {
          return y(d[1]);
        }).attr('height', function (d) {
          return y(d[0]) - y(d[1]);
        });

        g.append('g').attr('class', 'axis axis--x').attr('transform', 'translate(0,' + this.state.innerHeight + ')').call(d3.axisBottom(x).tickSize(1).tickPadding(6).tickFormat(function (d) {
          return data.xz[d];
        }));

        d3.selectAll('input').on('change', changed);

        timeout = d3.timeout(function () {
          d3.select('input[value="grouped"]').property('checked', true).dispatch('change');
        }, 2000);

        function changed() {
          timeout.stop();
          if (this.value === 'grouped') {
            transitionGrouped();
          } else {
            transitionStacked();
          }
        }

        function transitionGrouped() {
          y.domain([0, data.yMax]);

          rect.transition().duration(500).delay(function (d, i) {
            return i * 10;
          }).attr('x', function (d, i, g) {
            return x(i) + x.bandwidth() / data.n * g[0].parentNode.__data__.index;
          }).attr('width', x.bandwidth() / data.n).transition().attr('y', function (d) {
            return y(d[1] - d[0]);
          }).attr('height', function (d) {
            return y(0) - y(d[1] - d[0]);
          });
        }

        function transitionStacked() {
          y.domain([0, data.y1Max]);

          rect.transition().duration(500).delay(function (d, i) {
            return i * 10;
          }).attr('y', function (d) {
            return y(d[1]);
          }).attr('height', function (d) {
            return y(d[0]) - y(d[1]);
          }).transition().attr('x', function (d, i) {
            return x(i);
          }).attr('width', x.bandwidth());
        }
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement(
          'div',
          null,
          _react2.default.createElement(
            'form',
            null,
            _react2.default.createElement(
              'label',
              null,
              _react2.default.createElement('input', { type: 'radio', name: 'mode', value: 'grouped' }),
              'Grouped'
            ),
            _react2.default.createElement(
              'label',
              null,
              _react2.default.createElement('input', { type: 'radio', name: 'mode', value: 'stacked', defaultChecked: true }),
              'Stacked'
            )
          ),
          _react2.default.createElement('svg', {
            className: 'stackedgrouped',
            width: this.props.width,
            height: this.props.height })
        );
      }
    }]);

    return StackedGroupedBar;
  }(_react2.default.Component);

  StackedGroupedBar.propTypes = {
    data: _react2.default.PropTypes.object.isRequired,
    width: _react2.default.PropTypes.number,
    height: _react2.default.PropTypes.number
  };

  StackedGroupedBar.defaultProps = {
    width: 960,
    height: 500
  };

  exports.default = StackedGroupedBar;
});