(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3);
    global.circlePacking = mod.exports;
  }
})(this, function (exports, _react, _d) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var CirclePacking = function (_React$Component) {
    _inherits(CirclePacking, _React$Component);

    function CirclePacking(props) {
      _classCallCheck(this, CirclePacking);

      var _this = _possibleConstructorReturn(this, (CirclePacking.__proto__ || Object.getPrototypeOf(CirclePacking)).call(this, props));

      _this.state = {};
      return _this;
    }

    _createClass(CirclePacking, [{
      key: 'componentDidMount',
      value: function componentDidMount() {
        var height, width, svg, margin, diameter, g, color, pack;

        height = +this.props.height;

        width = +this.props.width;

        svg = d3.select(".circle-packing").attr('height', height).attr('width', width);

        margin = 20;

        diameter = +svg.attr("width");

        g = svg.append("g").attr("transform", "translate(" + diameter / 2 + "," + diameter / 2 + ")");

        color = d3.scaleLinear().domain([-1, 5]).range(["hsl(152,80%,80%)", "hsl(228,30%,40%)"]).interpolate(d3.interpolateHcl);

        pack = d3.pack().size([diameter - margin, diameter - margin]).padding(2);

        function genCirclePacking(root) {

          var focus, nodes, view, circle, text, node;

          root = d3.hierarchy(root).sum(function (d) {
            return d.size;
          }).sort(function (a, b) {
            return b.value - a.value;
          });

          focus = root;
          nodes = pack(root).descendants();

          circle = g.selectAll("circle").data(nodes).enter().append("circle").attr("class", function (d) {
            return d.parent ? d.children ? "node" : "node node--leaf" : "node node--root";
          }).style("fill", function (d) {
            return d.children ? color(d.depth) : 'white';
          }).on("click", function (d) {
            if (focus !== d) zoom(d), d3.event.stopPropagation();
          });

          text = g.selectAll("text").data(nodes).enter().append("text").attr("class", "label").style('font', '11px "Helvetica Neue", Helvetica, Arial, sans-serif').style('text-anchor', 'middle').style('text-shadow', '0 1px 0 #fff, 1px 0 0 #fff, -1px 0 0 #fff, 0 -1px 0 #fff').style("fill-opacity", function (d) {
            return d.parent === root ? 1 : 0;
          }).style("display", function (d) {
            return d.parent === root ? "inline" : "none";
          }).text(function (d) {
            return d.data.name;
          });

          node = g.selectAll("circle, text");

          g.selectAll("circle").style('cursor', 'pointer').on('mouseover', function (d) {
            var nodeSelection = d3.select(this).style('stroke', '#000').style('stroke-width', '1.5px');
          }).on('mouseout', function (d) {
            var nodeSelection = d3.select(this).style('stroke', '#000').style('stroke-width', '0px');
          });

          svg.style("background", color(-1)).on("click", function () {
            zoom(root);
          });

          zoomTo([root.x, root.y, root.r * 2 + margin]);

          function zoom(d) {
            var focus0 = focus;
            focus = d;

            var transition = d3.transition().duration(d3.event.altKey ? 7500 : 750).tween("zoom", function (d) {
              var i = d3.interpolateZoom(view, [focus.x, focus.y, focus.r * 2 + margin]);
              return function (t) {
                zoomTo(i(t));
              };
            });

            transition.selectAll("text").filter(function (d) {
              return d.parent === focus || this.style.display === "inline";
            }).style("fill-opacity", function (d) {
              return d.parent === focus ? 1 : 0;
            }).on("start", function (d) {
              if (d.parent === focus) this.style.display = "inline";
            }).on("end", function (d) {
              if (d.parent !== focus) this.style.display = "none";
            });
          }

          function zoomTo(v) {
            var k = diameter / v[2];view = v;
            node.attr("transform", function (d) {
              return "translate(" + (d.x - v[0]) * k + "," + (d.y - v[1]) * k + ")";
            });
            circle.attr("r", function (d) {
              return d.r * k;
            });
          }
        }
        genCirclePacking(this.props.data);
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement(
          'div',
          null,
          _react2.default.createElement('svg', {
            className: 'circle-packing' })
        );
      }
    }]);

    return CirclePacking;
  }(_react2.default.Component);

  exports.default = CirclePacking;
});