(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3);
    global.hierarchical = mod.exports;
  }
})(this, function (exports, _react, _d) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var HierarchicalEdgeBundle = function (_React$Component) {
    _inherits(HierarchicalEdgeBundle, _React$Component);

    function HierarchicalEdgeBundle(props) {
      _classCallCheck(this, HierarchicalEdgeBundle);

      return _possibleConstructorReturn(this, (HierarchicalEdgeBundle.__proto__ || Object.getPrototypeOf(HierarchicalEdgeBundle)).call(this, props));
    }

    _createClass(HierarchicalEdgeBundle, [{
      key: 'componentDidMount',
      value: function componentDidMount() {
        var diameter, radius, data, innerRadius, cluster, bundle, line, svg, link, node, hierarchy, hierarchy2, nodes, links;

        diameter = this.props.diameter;
        data = this.props.data;
        radius = diameter / 2;
        innerRadius = radius - 120;

        cluster = d3.cluster().size([360, innerRadius]);

        bundle = d3.curveBundle();

        line = d3.radialLine().angle(function (d) {
          return d.x;
        }).radius(function (d) {
          return d.y;
        }).curve(d3.curveBundle.beta(0.95));

        svg = d3.select("#hierarchical").append("svg").attr("width", diameter).attr("height", diameter).append("g").attr("transform", "translate(" + radius + "," + radius + ")");

        link = svg.append("g").selectAll(".link"), node = svg.append("g").selectAll(".node");

        hierarchy = packageHierarchy(data);

        hierarchy2 = d3.hierarchy(hierarchy);

        nodes = cluster(hierarchy2);

        links = packageImports(nodes);

        link = link.data(links).enter().append("path").attr("class", "link").attr("d", function (d) {
          return line(d.source.path(d.target));
        });

        node = node.data(nodes.descendants().filter(function (n) {
          return !n.children;
        })).enter().append("text").attr("class", "node").attr("dy", ".31em").attr("transform", function (d) {
          return "rotate(" + (d.x - 90) + ")translate(" + (d.y + 8) + ",0)" + (d.x < 180 ? "" : "rotate(180)");
        }).style("text-anchor", function (d) {
          return d.x < 180 ? "start" : "end";
        }).text(function (d) {
          return d.data.key;
        }).on("mouseover", mouseovered).on("mouseout", mouseouted);

        function mouseovered(d) {
          node.each(function (n) {
            n.target = n.source = false;
          });

          link.classed("link--target", function (l) {
            if (l.target === d) return l.source.source = true;
          }).classed("link--source", function (l) {
            if (l.source === d) return l.target.target = true;
          }).filter(function (l) {
            return l.target === d || l.source === d;
          }).each(function () {
            this.parentNode.appendChild(this);
          });

          node.classed("node--target", function (n) {
            return n.target;
          }).classed("node--source", function (n) {
            return n.source;
          });
        }

        function mouseouted(d) {
          link.classed("link--target", false).classed("link--source", false);

          node.classed("node--target", false).classed("node--source", false);
        }

        d3.select(self.frameElement).style("height", diameter + "px");

        // Lazily construct the package hierarchy from class names.
        function packageHierarchy(classes) {
          var map = {};

          function find(name, data) {
            var node = map[name],
                i;
            if (!node) {
              node = map[name] = data || { name: name, children: [] };
              if (name.length) {
                node.parent = find(name.substring(0, i = name.lastIndexOf(".")));
                node.parent.children.push(node);
                node.key = name.substring(i + 1);
              }
            }
            return node;
          }

          classes.forEach(function (d) {
            find(d.name, d);
          });

          return map[""];
        }

        // Return a list of imports for the given array of nodes.
        function packageImports(nodes) {
          var map = {};
          var imports = [];
          // Compute a map from name to node.
          nodes.descendants().forEach(function (d) {
            map[d.data.name] = d;
          });
          // For each import, construct a link from the source to target node.
          nodes.descendants().forEach(function (d) {
            if (d.data.imports) {
              d.data.imports.forEach(function (i) {
                imports.push({ source: map[d.data.name], target: map[i] });
              });
            }
          });
          return imports;
        }
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement('div', { id: 'hierarchical' });
      }
    }]);

    return HierarchicalEdgeBundle;
  }(_react2.default.Component);

  HierarchicalEdgeBundle.propTypes = {
    diameter: _react2.default.PropTypes.number,
    data: _react2.default.PropTypes.array.isRequired
  };

  HierarchicalEdgeBundle.defaultProps = {
    diameter: 960
  };

  exports.default = HierarchicalEdgeBundle;
});