(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3);
    global.scatterplot = mod.exports;
  }
})(this, function (exports, _react, _d) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var ScatterPlot = function (_React$Component) {
    _inherits(ScatterPlot, _React$Component);

    function ScatterPlot(props) {
      _classCallCheck(this, ScatterPlot);

      var _this = _possibleConstructorReturn(this, (ScatterPlot.__proto__ || Object.getPrototypeOf(ScatterPlot)).call(this, props));

      _this.state = {};
      return _this;
    }

    _createClass(ScatterPlot, [{
      key: 'componentDidMount',
      value: function componentDidMount() {

        var margin, width, height, svg, g, color;

        margin = { top: 20, right: 20, bottom: 30, left: 40 };
        width = this.props.width;
        height = this.props.height;

        svg = d3.select(".scatter-plot").append("svg").attr("width", width).attr("height", height);

        width = svg.attr("width") - margin.left - margin.right;
        height = svg.attr("height") - margin.top - margin.bottom, g = svg.append("g").attr("transform", "translate(" + margin.left + "," + margin.top + ")");

        color = d3.scaleOrdinal(d3.schemeCategory10);

        function genScatterPlot(data) {

          var x, y, legend;

          data.forEach(function (d) {
            var datum = d["sepalLength sepalWidth petalLength petalWidth species"].split(' ');
            d.sepalLength = +datum[0];
            d.sepalWidth = +datum[1];
            d.species = datum[4];
          });

          x = d3.scaleLinear().domain(d3.extent(data, function (d) {
            return d.sepalWidth;
          })).range([0, width]);

          y = d3.scaleLinear().domain(d3.extent(data, function (d) {
            return d.sepalLength;
          })).range([height, 0]);

          g.append("g").attr("transform", "translate(0," + height + ")").attr("class", "x-axis").call(d3.axisBottom(x).ticks(13, "s"));

          g.append("text").attr("transform", "translate(" + width / 2 + " ," + (height + margin.top + 7) + ")").style("text-anchor", "right").style('font', '10px sans-serif').text("Sepal Width (cm)");

          g.append("g").attr("transform", "translate(" + x + ",0)").attr("class", "y-axis").call(d3.axisLeft(y).ticks(9, 's'));

          g.append("text").attr("transform", "rotate(-90)").attr("y", 0 - margin.left).attr("x", 0 - height / 2).attr("dy", "1em").style("text-anchor", "middle").style('font', '10px sans-serif').text("Sepal Length (cm)");

          g.selectAll(".dot").data(data).enter().append("circle").attr("class", "dot").attr("r", 3.5).attr("cx", function (d) {
            return x(d.sepalWidth);
          }).attr("cy", function (d) {
            return y(d.sepalLength);
          }).style("fill", function (d) {
            return color(d.species);
          });

          legend = svg.selectAll(".legend").data(color.domain()).enter().append("g").attr("class", "legend").attr("transform", function (d, i) {
            return "translate(0," + i * 20 + ")";
          });

          legend.append("rect").attr("x", width - 18).attr("width", 18).attr("height", 18).style("fill", color);

          legend.append("text").attr("x", width - 24).attr("y", 9).attr("dy", ".35em").style("text-anchor", "end").text(function (d) {
            return d;
          });
        }

        genScatterPlot(this.props.data);
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement('div', { className: 'scatter-plot' });
      }
    }]);

    return ScatterPlot;
  }(_react2.default.Component);

  exports.default = ScatterPlot;
});