(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3);
    global.sequence = mod.exports;
  }
})(this, function (exports, _react, _d) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var explanationStyle = {
    position: 'absolute',
    top: '260px',
    left: '305px',
    width: '140px',
    textAlign: 'center',
    color: '#666',
    zIndex: -1,
    visibility: 'hidden'
  };

  var SequenceSunBurst = function (_React$Component) {
    _inherits(SequenceSunBurst, _React$Component);

    function SequenceSunBurst() {
      _classCallCheck(this, SequenceSunBurst);

      return _possibleConstructorReturn(this, (SequenceSunBurst.__proto__ || Object.getPrototypeOf(SequenceSunBurst)).call(this));
    }

    _createClass(SequenceSunBurst, [{
      key: 'componentDidMount',
      value: function componentDidMount() {
        // Dimensions of sunburst.
        var width, height, data, radius, x, y, b, colors, totalSize, vis, partition, arc;

        width = this.props.width;
        height = this.props.height;
        data = this.props.data;
        radius = Math.min(width, height) / 2;

        x = d3.scaleLinear().range([0, 2 * Math.PI]);

        y = d3.scaleLinear().range([0, radius]);

        // Breadcrumb dimensions: width, height, spacing, width of tip/tail.
        b = { w: 75, h: 30, s: 3, t: 10 };

        colors = d3.scaleOrdinal(d3.schemeCategory20);

        // Total size of all segments; we set this later, after loading the data.
        totalSize = 0;

        vis = d3.select("#chart").append("svg:svg").attr("width", width).attr("height", height).append("svg:g").attr("id", "container").attr("transform", "translate(" + width / 2 + "," + height / 2 + ")");

        partition = d3.partition();

        arc = d3.arc().startAngle(function (d) {
          return Math.max(0, Math.min(2 * Math.PI, x(d.x0)));
        }).endAngle(function (d) {
          return Math.max(0, Math.min(2 * Math.PI, x(d.x1)));
        }).innerRadius(function (d) {
          return Math.max(0, y(d.y0));
        }).outerRadius(function (d) {
          return Math.max(0, y(d.y1));
        });

        createVisualization(data);

        // Main function to draw and set up the visualization, once we have the data.
        function createVisualization(json) {

          // Basic setup of page elements.
          initializeBreadcrumbTrail();
          drawLegend();
          d3.select("#togglelegend").on("click", toggleLegend);

          // Bounding circle underneath the sunburst, to make it easier to detect
          // when the mouse leaves the parent g.
          vis.append("svg:circle").attr("r", radius).style("opacity", 0);

          // For efficiency, filter nodes to keep only those large enough to see.
          var root = d3.hierarchy(json);
          root.sum(function (d) {
            return d.size;
          });

          var nodes = partition(root).descendants();
          // filtering filters out over half of nodes
          /*          .filter(function(d) {
                    var dx = d.x1 - d.x0;
                    return (dx > 0.005); // 0.005 radians = 0.29 degrees
                    });*/

          // Determines max depth when entering data to paths
          var maxDepth = 0;
          var path = vis.data(nodes).selectAll("path").data(nodes).enter().append("svg:path").attr("display", function (d) {
            return d.depth ? null : "none";
          }).attr("d", arc).attr("fill-rule", "evenodd").style("fill", function (d) {
            if (d.depth > maxDepth) {
              maxDepth = d.depth;
            }
            return colors((d.children ? d : d.parent).data.name);
          }).style("opacity", 1).on("mouseover", mouseover);

          // Bounding inner circle based on depth of elements
          var innerG = vis.append("g");

          var innerBound = innerG.append("circle").attr("r", radius / (maxDepth + 1)).attr("id", "innerBound").style("opacity", 0);

          var innerText = innerG.append("text").attr("id", "percentage").attr("x", -20).attr("y", 0).text("");

          // Add the mouseleave handler to the bounding circle.
          d3.select("#container").on("mouseleave", mouseleave);

          // Get total size of the tree = value of root node from partition.
          totalSize = path.node().__data__.value;
        };

        // Fade all but the current sequence, and show it in the breadcrumb trail.
        function mouseover(d) {

          var percentage = (100 * d.value / totalSize).toPrecision(3);
          var percentageString = percentage + "%";
          if (percentage < 0.1) {
            percentageString = "< 0.1%";
          }

          d3.select("#percentage").text(percentageString);

          d3.select("#explanation").style("visibility", "");

          var sequenceArray = getAncestors(d);
          updateBreadcrumbs(sequenceArray, percentageString);

          // Fade all the segments.
          d3.selectAll("path").style("opacity", 0.3);

          // Then highlight only those that are an ancestor of the current segment.
          vis.selectAll("path").filter(function (node) {
            return sequenceArray.indexOf(node) >= 0;
          }).style("opacity", 1);
        }

        // Restore everything to full opacity when moving off the visualization.
        function mouseleave(d) {

          // Hide the breadcrumb trail
          d3.select("#trail").style("visibility", "hidden");

          // Deactivate all segments during transition.
          d3.selectAll("path").on("mouseover", null);

          // Transition each segment to full opacity and then reactivate it.
          d3.selectAll("path").transition().duration(1000).style("opacity", 1).on("end", function () {
            d3.select(this).on("mouseover", mouseover);
            d3.select("#percentage").text("");
          });

          d3.select("#explanation").style("visibility", "hidden");
        }

        // Given a node in a partition layout, return an array of all of its ancestor
        // nodes, highest first, but excluding the root.
        function getAncestors(node) {
          var path = [];
          var current = node;
          while (current.parent) {
            path.unshift(current);
            current = current.parent;
          }
          return path;
        }

        function initializeBreadcrumbTrail() {
          // Add the svg area.
          var trail = d3.select("#sequence").append("svg:svg").attr("width", width).attr("height", 50).attr("id", "trail");
          // Add the label at the end, for the percentage.
          trail.append("svg:text").attr("id", "endlabel").style("fill", "#000");
        }

        // Generate a string that describes the points of a breadcrumb polygon.
        function breadcrumbPoints(d, i) {
          var points = [];
          points.push("0,0");
          points.push(b.w + ",0");
          points.push(b.w + b.t + "," + b.h / 2);
          points.push(b.w + "," + b.h);
          points.push("0," + b.h);
          if (i > 0) {
            // Leftmost breadcrumb; don't include 6th vertex.
            points.push(b.t + "," + b.h / 2);
          }
          return points.join(" ");
        }

        var context = this;
        // Update the breadcrumb trail to show the current sequence and percentage.
        function updateBreadcrumbs(nodeArray, percentageString) {
          // Data join; key function combines name and depth (= position in sequence).
          var g = d3.select("#trail").selectAll("g").data(nodeArray, function (d) {
            return d.data.name + d.depth;
          });

          // Add breadcrumb and label for entering nodes.
          var entering = g.enter().append("g");

          entering.append("svg:polygon").attr("points", breadcrumbPoints).style("fill", function (d) {
            return colors((d.children ? d : d.parent).data.name);
          });

          entering.append("svg:text").attr("x", (b.w + b.t) / 2).attr("y", b.h / 2).attr("dy", "0.35em").attr("text-anchor", "middle").text(function (d) {
            return d.data.name;
          });

          // Set position for entering and updating nodes.
          d3.select("#trail").selectAll("g").attr("transform", function (d, i) {
            return "translate(" + (d.depth - 1) * (b.w + b.s) + ", 0)";
          });

          // Remove exiting nodes.
          g.exit().remove();

          // Now move and update the percentage at the end.
          d3.select("#trail").select("#endlabel").attr("x", (nodeArray.length + 0.5) * (b.w + b.s)).attr("y", b.h / 2).attr("dy", "0.35em").attr("text-anchor", "middle").text(percentageString);

          // Make the breadcrumb trail visible, if it's hidden.
          d3.select("#trail").style("visibility", "");
        }

        function drawLegend() {

          // Dimensions of legend item: width, height, spacing, radius of rounded rect.
          var li = {
            w: 75, h: 30, s: 3, r: 3
          };

          var legend = d3.select("#legend").append("svg:svg").attr("width", li.w).attr("height", d3.keys(colors).length * (li.h + li.s));

          var g = legend.selectAll("g").data(d3.entries(colors)).enter().append("svg:g").attr("transform", function (d, i) {
            return "translate(0," + i * (li.h + li.s) + ")";
          });

          g.append("svg:rect").attr("rx", li.r).attr("ry", li.r).attr("width", li.w).attr("height", li.h).style("fill", function (d) {
            return d.value;
          });

          g.append("svg:text").attr("x", li.w / 2).attr("y", li.h / 2).attr("dy", "0.35em").attr("text-anchor", "middle").text(function (d) {
            return d.key;
          });
        }

        function toggleLegend() {
          var legend = d3.select("#legend");
          if (legend.style("visibility") == "hidden") {
            legend.style("visibility", "");
          } else {
            legend.style("visibility", "hidden");
          }
        }
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement(
          'div',
          null,
          _react2.default.createElement(
            'div',
            { id: 'main' },
            _react2.default.createElement('div', { id: 'sequence' }),
            _react2.default.createElement('div', { id: 'chart' })
          ),
          _react2.default.createElement(
            'div',
            { id: 'sidebar' },
            _react2.default.createElement('input', { type: 'checkbox', id: 'togglelegend' }),
            ' Legend',
            _react2.default.createElement('br', null),
            _react2.default.createElement('div', { id: 'legend', style: { visibility: "hidden" } })
          )
        );
      }
    }]);

    return SequenceSunBurst;
  }(_react2.default.Component);

  SequenceSunBurst.propTypes = {
    width: _react2.default.PropTypes.number,
    height: _react2.default.PropTypes.number,
    data: _react2.default.PropTypes.array.isRequired
  };

  SequenceSunBurst.defaultProps = {
    width: 750,
    height: 600
  };

  exports.default = SequenceSunBurst;
});