(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3);
    global.voronoi = mod.exports;
  }
})(this, function (exports, _react, _d) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var Voronoi = function (_React$Component) {
    _inherits(Voronoi, _React$Component);

    function Voronoi(props) {
      _classCallCheck(this, Voronoi);

      var _this = _possibleConstructorReturn(this, (Voronoi.__proto__ || Object.getPrototypeOf(Voronoi)).call(this, props));

      _this.state = {
        voronoi: null,
        polygon: null,
        link: null,
        site: null,
        options: {
          'polyFill': _this.props.polyFill || 'none'

        }
      };
      _this.redraw = _this.redraw.bind(_this);
      _this.moved = _this.moved.bind(_this);
      return _this;
    }

    _createClass(Voronoi, [{
      key: 'componentDidMount',
      value: function componentDidMount() {
        var svg, voronoi, polygon, link, site, yo;

        svg = d3.select('.voronoi');
        voronoi = d3.voronoi().extent([[-1, -1], [960 + 1, 500 + 1]]);
        polygon = svg.append('g').attr('class', 'polygon').selectAll('path').data(voronoi.polygons(this.props.data)).enter().append('path').attr('fill', 'none').attr('stroke', '#000').call(this.redrawPolygon);
        link = svg.append('g').attr('class', 'links').selectAll('line').data(voronoi.links(this.props.data)).enter().append('line').attr('stroke', '#000').attr('stroke-opacity', '0.2').call(this.redrawLink);
        site = svg.append('g').attr('class', 'sites').selectAll('circle').data(this.props.data).enter().append('circle').attr('r', 2.5).attr('fill', '#000').attr('stroke', '#fff').call(this.redrawSite);

        this.setState({
          voronoi: voronoi,
          polygon: polygon,
          link: link,
          site: site
        });
      }
    }, {
      key: 'moved',
      value: function moved(e) {
        var temp = this.props.data;
        temp[0] = [e.clientX, e.clientY];
        this.setState({ dataSet: temp });
        this.redraw();
        d3.select('.voronoi').select('.polygon').select('path').attr('fill', '#f00');
        d3.select('.voronoi').select('.sites').select('circle').attr('fill', '#fff');
      }
    }, {
      key: 'redraw',
      value: function redraw() {
        var diagram = this.state.voronoi(this.props.data);
        this.setState({ polygon: this.state.polygon.data(diagram.polygons()).call(this.redrawPolygon) });
        this.setState({ link: this.state.link.data(diagram.links()) });
        this.setState({ link: this.state.link.exit().remove() });
        this.setState({ link: this.state.link.enter().append('line').merge(this.state.link).call(this.redrawLink) });
        this.setState({ site: this.state.site.data(this.props.data).call(this.redrawSite) });
      }
    }, {
      key: 'redrawPolygon',
      value: function redrawPolygon(polygon) {
        polygon.attr('d', function (d) {
          return d ? "M" + d.join("L") + "Z" : null;
        });
      }
    }, {
      key: 'redrawLink',
      value: function redrawLink(link) {
        link.attr('x1', function (d) {
          return d.source[0];
        }).attr('y1', function (d) {
          return d.source[1];
        }).attr('x2', function (d) {
          return d.target[0];
        }).attr('y2', function (d) {
          return d.target[1];
        });
      }
    }, {
      key: 'redrawSite',
      value: function redrawSite(site) {
        site.attr('cx', function (d) {
          return d[0];
        }).attr('cy', function (d) {
          return d[1];
        });
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement(
          'div',
          null,
          _react2.default.createElement('svg', {
            width: this.props.width,
            height: this.props.height,
            className: 'voronoi',
            onMouseMove: this.moved })
        );
      }
    }]);

    return Voronoi;
  }(_react2.default.Component);

  Voronoi.propTypes = {
    data: _react2.default.PropTypes.array.isRequired,
    width: _react2.default.PropTypes.number,
    height: _react2.default.PropTypes.number
  };

  Voronoi.defaultProps = {
    width: 960,
    height: 500
  };

  exports.default = Voronoi;
});