(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react', 'd3', 'topojson', './../assets/usamap.json'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react'), require('d3'), require('topojson'), require('./../assets/usamap.json'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.react, global.d3, global.topojson, global.usamap);
    global.zoomableMap = mod.exports;
  }
})(this, function (exports, _react, _d, _topojson, _usamap) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _react2 = _interopRequireDefault(_react);

  var d3 = _interopRequireWildcard(_d);

  var topojson = _interopRequireWildcard(_topojson);

  var _usamap2 = _interopRequireDefault(_usamap);

  function _interopRequireWildcard(obj) {
    if (obj && obj.__esModule) {
      return obj;
    } else {
      var newObj = {};

      if (obj != null) {
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
        }
      }

      newObj.default = obj;
      return newObj;
    }
  }

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var ZoomableMap = function (_React$Component) {
    _inherits(ZoomableMap, _React$Component);

    function ZoomableMap(props) {
      _classCallCheck(this, ZoomableMap);

      var _this = _possibleConstructorReturn(this, (ZoomableMap.__proto__ || Object.getPrototypeOf(ZoomableMap)).call(this, props));

      _this.state = {
        centered: null,
        options: {
          // 'polyFill': this.props.polyFill || 'none',

        }
      };
      _this.click = _this.click.bind(_this);

      return _this;
    }

    _createClass(ZoomableMap, [{
      key: 'componentDidMount',
      value: function componentDidMount() {
        var svg, projection, path, background, g;

        svg = d3.select('.zoomableMap');
        projection = d3.geoAlbersUsa().scale(1070).translate([480, 250]);
        path = d3.geoPath().projection(projection);
        background = svg.append('rect').attr('width', this.props.width).attr('height', this.props.height).attr('fill', 'none').attr('pointer-events', 'all').on('click', this.click);
        g = svg.append('g').attr('id', 'temp');

        g.append('g').attr('id', 'states').attr('fill', '#aaa').selectAll('path').data(topojson.feature(_usamap2.default, _usamap2.default.objects.states).features).enter().append('path').attr('d', path).on('click', this.click);

        g.append('path').datum(topojson.mesh(_usamap2.default, _usamap2.default.objects.states, function (a, b) {
          return a !== b;
        })).attr('id', 'state-borders').attr('d', path).attr('fill', 'none').attr('stroke', '#fff').attr('stroke-width', '1.5px').attr('stroke-linejoin', 'round').attr('stroke-linecap', 'round').attr('pointer-events', 'none');

        this.setState({
          projection: projection,
          path: path,
          g: g
        });
      }
    }, {
      key: 'click',
      value: function click(d) {
        var _this2 = this;

        var x, y, k;

        if (d && this.state.centered !== d) {
          this.setState({ centered: d });
          var centroid = this.state.path.centroid(d);
          x = centroid[0];
          y = centroid[1];
          k = 4;
        } else {
          this.setState({ centered: null });
          x = this.props.width / 2;
          y = this.props.height / 2;
          k = 1;
          d3.select('#temp').selectAll('path').classed('active', false);
        }

        this.state.g.selectAll('path').classed('active', this.state.centered && function (x) {
          return x === _this2.state.centered;
        });

        d3.selectAll('#states path').attr('fill', '#aaa');
        d3.select('.active').attr('fill', 'orange');

        d3.select('#temp').transition().duration(750).attr('transform', 'translate(' + this.props.width / 2 + ',' + this.props.height / 2 + ')scale(' + k + ')translate(' + -x + ',' + -y + ')').style('stroke-width', 1.5 / k + 'px');
      }
    }, {
      key: 'render',
      value: function render() {
        return _react2.default.createElement(
          'div',
          null,
          _react2.default.createElement('svg', {
            width: this.props.width,
            height: this.props.height,
            className: 'zoomableMap'
          })
        );
      }
    }]);

    return ZoomableMap;
  }(_react2.default.Component);

  ZoomableMap.propTypes = {
    width: _react2.default.PropTypes.number,
    height: _react2.default.PropTypes.number
  };

  ZoomableMap.defaultProps = {
    width: 960,
    height: 500
  };

  exports.default = ZoomableMap;
});